<!doctype HTML>

<html>
	<head>
		<meta charset=utf-8>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">

		<title>django-embed-video example project</title>

    <link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.0.0/css/bootstrap.min.css">
    <style>
    body {
      font-size: 32px;
    }

    h1 {
      font-size: 64px;
      margin: 20px 5px;
    }
    </style>
	</head>

	<body>
    <div class="container">
      {% block content %}{% endblock %}
    </div>

	</body>
</html>

